SystemOrganization addCategory: #'Zinc-SSO-OAuth1-Core'!

Object subclass: #ZnOAuth1AuthorizationCalculator
	instanceVariableNames: 'consumerKey consumerSecret signatureMethod version nonce timestamp token tokenSecret callback verifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnOAuth1AuthorizationCalculator commentStamp: '<historical>' prior: 0!
Utility class to calculate the correct Authorization header for OAuth1 requests.!

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 18:52'!
callback
	^ callback! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 18:52'!
callback: anObject
	callback := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
consumerKey
	^ consumerKey! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
consumerKey: anObject
	consumerKey := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
consumerSecret
	^ consumerSecret! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
consumerSecret: anObject
	consumerSecret := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'api' stamp: 'JanVanDeSandt 12/26/2012 18:43'!
getAuthorizationHeader: aRequest
	"Create the OAuth1 Authorization header. See: http://code.google.com/apis/accounts/docs/OAuth_ref.html "

	| oauthParameters baseString signatureKey encoder |
	
	oauthParameters := self getOAuthParameters.
	baseString := self getBaseString: aRequest with: oauthParameters.
	signatureKey := self getSignatureKey.
	encoder := ZnPercentEncoder new.
	
	self signatureMethod = 'HMAC-SHA1'
		ifTrue: [
			oauthParameters add: 'oauth_signature' -> (ZnSSOMessage signStringSHA1: baseString usingSecret: signatureKey) ]
		ifFalse: [
			self error: 'Unsupported signature method' ].
		
	^String streamContents: [ :stream |
		stream nextPutAll: 'OAuth '.
		oauthParameters 
			do: [ :assoc |
				stream
					nextPutAll: assoc key ;
					nextPutAll: '="'.
				encoder encode: assoc value readStream to: stream.
				stream nextPutAll: '"' ]
			separatedBy: [ stream nextPutAll: ',' ] ]! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'actions' stamp: 'JanVanDeSandt 12/26/2012 23:01'!
getBaseString: aRequest with: anOAuthParameters
	"Create the OAuth1 baseString"

	| params paramsString percentEncoder |
	
	percentEncoder := ZnPercentEncoder new.
	
	params := OrderedCollection new
		addAll: anOAuthParameters ;
		yourself.
	aRequest uri queryDo: [ :key :value | params add: key -> value ].
	(aRequest method = 'POST' and: [ aRequest entity isKindOf: ZnApplicationFormUrlEncodedEntity ] )
		ifTrue: [
			aRequest entity fields keysAndValuesDo: [ :key :value | params add: key -> value ] ].
	
	params sort: [ :a1 :a2 |
		a1 key = a2 key
			ifTrue: [ a1 value < a2 value ]
			ifFalse: [ a1 key < a2 key ] ].
		
	paramsString := String streamContents: [ :stream |
		params 
			do: [ :assoc |
				stream
					nextPutAll: assoc key ;
					nextPut: $=.
				percentEncoder encode: assoc value readStream to: stream ]
			separatedBy: [ stream nextPutAll: '&' ] ].
		
	^String streamContents: [ :stream | | baseUrl |
		stream
			nextPutAll: aRequest method ;
			nextPut: $&.
		baseUrl := aRequest uri copy
			query: nil ;
			fragment: nil ;
			asString.
		percentEncoder encode: baseUrl readStream to: stream. 	
		stream nextPut: $&.
		percentEncoder encode: paramsString readStream to:  stream ]! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'actions' stamp: 'JanVanDeSandt 12/26/2012 21:50'!
getOAuthParameters

	| list |
	
	list := OrderedCollection new
		add: 'oauth_consumer_key' -> self consumerKey ;
		add: 'oauth_nonce' -> self nonce ;
		add: 'oauth_signature_method' -> self signatureMethod ;
		add: 'oauth_timestamp' -> self timestamp ;
		add: 'oauth_version' -> self version ;
		yourself.
		
	self token notNil
		ifTrue: [ list add: 'oauth_token' -> self token ].
	self callback notNil
		ifTrue: [ list add: 'oauth_callback' -> self callback ].
	self verifier notNil
		ifTrue: [ list add: 'oauth_verifier' -> self verifier ].
	^ list! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'actions' stamp: 'JanVanDeSandt 12/26/2012 16:32'!
getSignatureKey
	"Create the OAuth1 signature key. "

	| percentEncoder |

	percentEncoder := ZnPercentEncoder new.

	^String streamContents: [ :stream |
		percentEncoder encode: self consumerSecret readStream to: stream.
		stream nextPut: $&.
		percentEncoder encode: (self tokenSecret ifNil: [ '' ]) readStream to: stream ]! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/26/2012 16:29'!
initialize

	super initialize.
	self setUp! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
nonce
	^ nonce! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
nonce: anObject
	nonce := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/26/2012 16:28'!
setUp

	| begin secondsSinceEpoch |
	
	begin := DateAndTime year: 1970 day: 1 hour: 0 minute: 0 second: 0 offset: Duration zero. 
	secondsSinceEpoch := (DateAndTime now asUTC - begin) asSeconds.
	
	self version: '1.0'.
	self signatureMethod: 'HMAC-SHA1'.
	self timestamp: secondsSinceEpoch asString.
	self nonce: UUID new asString! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
signatureMethod
	^ signatureMethod! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
signatureMethod: anObject
	signatureMethod := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
timestamp
	^ timestamp! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
timestamp: anObject
	timestamp := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
token
	^ token! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
token: anObject
	token := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
tokenSecret
	^ tokenSecret! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
tokenSecret: anObject
	tokenSecret := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 21:49'!
verifier
	^ verifier! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 21:49'!
verifier: anObject
	verifier := anObject! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
version
	^ version! !

!ZnOAuth1AuthorizationCalculator methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/26/2012 16:31'!
version: anObject
	version := anObject! !

Object subclass: #ZnOAuth1ConsumerData
	instanceVariableNames: 'getRequestTokenUrl authorizeTokenUrl getAccessTokenUrl consumer consumerSecret accessTokenClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnOAuth1ConsumerData class methodsFor: 'instance creation' stamp: 'jvds 2/13/2011 22:13'!
newForGoogle

	^self new
		getRequestTokenUrl: 'https://www.google.com/accounts/OAuthGetRequestToken' ;
		authorizeTokenUrl: 'https://www.google.com/accounts/OAuthAuthorizeToken' ;
		getAccessTokenUrl: 'https://www.google.com/accounts/OAuthGetAccessToken' ;
		yourself! !

!ZnOAuth1ConsumerData class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/23/2012 15:12'!
newForTwitter

	^self new
		getRequestTokenUrl: 'https://api.twitter.com/oauth/request_token' ;
		authorizeTokenUrl: 'https://api.twitter.com/oauth/authorize' ;
		getAccessTokenUrl: 'https://api.twitter.com/oauth/access_token' ;
		accessTokenClass:  ZnOAuth1TwitterAccessToken ;
		yourself! !

!ZnOAuth1ConsumerData class methodsFor: 'instance creation' stamp: 'jvds 6/2/2011 16:31'!
newForTwitterAuthentication
	"Twitter variant for authentication instead of authorization."

	^self newForTwitter 
		authorizeTokenUrl: 'https://api.twitter.com/oauth/authenticate' ;
		yourself! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/23/2012 15:12'!
accessTokenClass
	^ accessTokenClass ifNil: [ accessTokenClass := ZnOAuth1Token ]! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 6/2/2011 16:29'!
accessTokenClass: anObject
	accessTokenClass := anObject! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
authorizeTokenUrl
	^ authorizeTokenUrl! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
authorizeTokenUrl: anObject
	authorizeTokenUrl := anObject! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
consumer
	^ consumer! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
consumer: anObject
	consumer := anObject! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
consumerSecret
	^ consumerSecret! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
consumerSecret: anObject
	consumerSecret := anObject! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
getAccessTokenUrl
	^ getAccessTokenUrl! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
getAccessTokenUrl: anObject
	getAccessTokenUrl := anObject! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
getRequestTokenUrl
	^ getRequestTokenUrl! !

!ZnOAuth1ConsumerData methodsFor: 'accessing' stamp: 'jvds 1/1/2011 21:01'!
getRequestTokenUrl: anObject
	getRequestTokenUrl := anObject! !

!ZnOAuth1ConsumerData methodsFor: 'testing' stamp: 'jvds 2/13/2011 22:14'!
hasKeys

	^ self consumer notNil and: [ self consumerSecret notNil ]! !

!ZnOAuth1ConsumerData methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 12/26/2012 18:49'!
newCalculator

	^ ZnOAuth1AuthorizationCalculator new
		consumerKey: self consumer ;
		consumerSecret: self consumerSecret ;
		yourself! !

Object subclass: #ZnOAuth1Service
	instanceVariableNames: 'providerAccount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnOAuth1Service methodsFor: 'actions' stamp: 'jvds 3/4/2011 21:32'!
executeRequest: aRequest token: anAccessToken
	" Execute the HTTP request <aRequest>. Use <anAccessToken> to set the OAuth Authorization header."

	^self executeRequest: aRequest token: anAccessToken followRedirects: true! !

!ZnOAuth1Service methodsFor: 'actions' stamp: 'JanVanDeSandt 12/26/2012 21:59'!
executeRequest: aRequest token: anAccessToken followRedirects: aBoolean
	"Execute the HTTP request <aRequest>. Use <anAccessToken> to set the OAuth Authorization header."

	| calculator url response |

	calculator := self providerAccount newCalculator.
	anAccessToken notNil
		ifTrue: [
			calculator 
				token: anAccessToken value;
				tokenSecret: anAccessToken secret ].
	
	aRequest setAuthorization: (calculator getAuthorizationHeader: aRequest).
	response := self httpClient 
		request: aRequest ;
		execute ;
		response.
	
	(aBoolean and: [ response isRedirect ])
		ifTrue: [ 
			| newRequest |
			newRequest := aRequest copy
				url: (response headers at: 'Location') ;
				yourself.
			^self executeRequest: newRequest token: anAccessToken ].
	^response! !

!ZnOAuth1Service methodsFor: 'actions' stamp: 'jvds 2/10/2011 22:19'!
getAccessToken: aToken
	"The final step: Use the token <aToken> that was authorized by the user to get the access token."

	^self getAccessToken: aToken verifier: nil! !

!ZnOAuth1Service methodsFor: 'actions' stamp: 'JanVanDeSandt 12/26/2012 21:56'!
getAccessToken: aToken verifier: aVerifier
	"The final step: Use the token <aToken> that was authorized by the user to get the access token."

	| calculator request response responseProperties |
	
	calculator := self providerAccount newCalculator
		token: aToken value ;
		tokenSecret: aToken secret ;
		verifier: aVerifier ;
		yourself.

	request := ZnRequest get: self providerAccount getAccessTokenUrl.
	request
		setAccept: '*/*' ;
		setAuthorization: (calculator getAuthorizationHeader: request).

	response := self httpClient
		request: request ;
		get.
	
	responseProperties := self resourceMetaUtils parseQueryFrom: response readStream.

	^self providerAccount accessTokenClass newFromDictionary: responseProperties ! !

!ZnOAuth1Service methodsFor: 'actions' stamp: 'jvds 2/10/2011 22:15'!
getRequestTokenFor: aCallbackUrl
	"The first step is to get a token and token_secret for <aCallbackUrl>."

	^self getRequestTokenFor: aCallbackUrl with: nil! !

!ZnOAuth1Service methodsFor: 'actions' stamp: 'JanVanDeSandt 12/26/2012 21:55'!
getRequestTokenFor: aCallbackUrl with: extraParameters
	"The first step is to get a token and token_secret for <aCallbackUrl>."

	| calculator request response responseProperties |
	
	calculator := self providerAccount newCalculator
		callback: aCallbackUrl asString ;
		yourself.
	
	request := ZnRequest get: self providerAccount getRequestTokenUrl.
	extraParameters notNil
		ifTrue: [ extraParameters keysAndValuesDo: [ :key :value | request uri queryAt: key put: value ] ].
	request
		setAccept: '*/*' ;
		setAuthorization: (calculator getAuthorizationHeader: request) ;
		yourself.

	response := self httpClient
		request: request ;
		get.

	responseProperties := self resourceMetaUtils parseQueryFrom: response readStream.
	(responseProperties at: 'oauth_callback_confirmed') = 'true'
		ifFalse: [ self error: 'Callback not confirmed' ].
		
	^ZnOAuth1Token newFromDictionary: responseProperties! !

!ZnOAuth1Service methodsFor: 'testing' stamp: 'jvds 2/13/2011 22:22'!
hasKeys
	^self providerAccount hasKeys! !

!ZnOAuth1Service methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 16:11'!
httpClient

	^ZnClient new
		dontFollowRedirects ;
		yourself! !

!ZnOAuth1Service methodsFor: 'actions' stamp: 'JanVanDeSandt 12/23/2012 17:20'!
httpGet: anUrl using: aToken

	| response |
	
	response := self executeRequest: (ZnRequest get: anUrl) token: aToken.
	response isSuccess
		ifFalse: [ self error: 'httpGet failed' ].

	^response contents! !

!ZnOAuth1Service methodsFor: 'actions' stamp: 'JanVanDeSandt 12/27/2012 10:41'!
loginUrlFor: aToken
	"Answer the url to which we should redirect the user. If the user grants us access the provider will redirect
	the user to the callbackUrl. The redirect will contain a new token which we can use to get an access token."

	^self providerAccount authorizeTokenUrl asZnUrl
		queryAt: 'oauth_token' put: aToken value ;
		yourself! !

!ZnOAuth1Service methodsFor: 'accessing' stamp: 'jvds 1/2/2011 14:57'!
providerAccount
	^ providerAccount! !

!ZnOAuth1Service methodsFor: 'accessing' stamp: 'jvds 1/2/2011 14:57'!
providerAccount: anObject
	providerAccount := anObject! !

!ZnOAuth1Service methodsFor: 'resources' stamp: 'JanVanDeSandt 12/26/2012 19:54'!
resourceMetaUtils

	^ZnResourceMetaUtils! !

Object subclass: #ZnOAuth1Token
	instanceVariableNames: 'value secret sessionid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnOAuth1Token class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/23/2012 17:24'!
newFrom: aString

	^self newFromDictionary: (ZnResourceMetaUtils parseQueryFrom: aString readStream)! !

!ZnOAuth1Token class methodsFor: 'instance creation' stamp: 'jvds 6/2/2011 16:32'!
newFromDictionary: aDictionary

	^self new
		value: (aDictionary at: 'oauth_token');
		secret: (aDictionary at: 'oauth_token_secret');
		yourself
! !

!ZnOAuth1Token methodsFor: 'accessing' stamp: 'jvds 1/3/2011 15:39'!
secret
	^ secret! !

!ZnOAuth1Token methodsFor: 'accessing' stamp: 'jvds 1/3/2011 15:39'!
secret: anObject
	secret := anObject! !

!ZnOAuth1Token methodsFor: 'accessing' stamp: 'jvds 3/4/2011 23:17'!
sessionid
	^ sessionid! !

!ZnOAuth1Token methodsFor: 'accessing' stamp: 'jvds 3/4/2011 23:17'!
sessionid: anObject
	sessionid := anObject! !

!ZnOAuth1Token methodsFor: 'accessing' stamp: 'jvds 1/3/2011 15:39'!
value
	^ value! !

!ZnOAuth1Token methodsFor: 'accessing' stamp: 'jvds 1/3/2011 15:39'!
value: anObject
	value := anObject! !

ZnOAuth1Token subclass: #ZnOAuth1TwitterAccessToken
	instanceVariableNames: 'userId screenName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnOAuth1TwitterAccessToken class methodsFor: 'instance creation' stamp: 'jvds 6/2/2011 16:37'!
newFromDictionary: aDictionary

	^(super newFromDictionary:  aDictionary)
		userId: (aDictionary at: 'user_id') ;
		screenName: (aDictionary at: 'screen_name') ;
		yourself
! !

!ZnOAuth1TwitterAccessToken methodsFor: 'accessing' stamp: 'jvds 6/2/2011 16:27'!
screenName
	^ screenName! !

!ZnOAuth1TwitterAccessToken methodsFor: 'accessing' stamp: 'jvds 6/2/2011 16:27'!
screenName: anObject
	screenName := anObject! !

!ZnOAuth1TwitterAccessToken methodsFor: 'accessing' stamp: 'jvds 6/2/2011 16:27'!
userId
	^ userId! !

!ZnOAuth1TwitterAccessToken methodsFor: 'accessing' stamp: 'jvds 6/2/2011 16:27'!
userId: anObject
	userId := anObject! !

Object subclass: #ZnOAuth1UserAccess
	instanceVariableNames: 'oauth1Service accessToken'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

ZnOAuth1UserAccess subclass: #ZnOAuth1GoogleUserAccess
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnOAuth1GoogleUserAccess methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 12/23/2012 15:40'!
getContacts

	| url result |
	
	url := ZnUrl fromString: 'https://www.google.com/m8/feeds/contacts/default/full'.
	result := self oauth1Service httpGet: url using: self accessToken.
	
	^result asString! !

ZnOAuth1UserAccess subclass: #ZnOAuth1TwitterUserAccess
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnOAuth1TwitterUserAccess methodsFor: 'twitter-api' stamp: 'JanVanDeSandt 12/26/2012 22:22'!
accountVerifyCredentials
	"https://dev.twitter.com/docs/api/1.1/get/account/verify_credentials "

	| url response json |

	url := ZnUrl fromString: 'https://api.twitter.com/1.1/account/verify_credentials.json'.
		
	response := (self oauth1Service httpGet: url using: self accessToken) asString.
	json := NeoJSONReader fromString: response.
	
	^json
! !

!ZnOAuth1TwitterUserAccess methodsFor: 'twitter-api' stamp: 'JanVanDeSandt 12/23/2012 15:42'!
getFriendsTimeline

	| url response json |

	url := ZnUrl fromString: 'http://api.twitter.com/1/statuses/friends_timeline.json'.
		
	response := (self oauth1Service httpGet: url using: self accessToken) asString.
	json := NeoJSONReader fromString: response.
	
	^json
! !

!ZnOAuth1TwitterUserAccess methodsFor: 'twitter-api' stamp: 'JanVanDeSandt 12/27/2012 11:25'!
statusesUpdate: aString
	"https://dev.twitter.com/docs/api/1.1/post/statuses/update"

	| request response json |

	request := ZnRequest post: 'https://api.twitter.com/1.1/statuses/update.json'.
	request entity: (ZnApplicationFormUrlEncodedEntity new
		at: 'status' put: aString ;
		yourself).
		
	response := self oauth1Service executeRequest: request token: self accessToken.
	response isSuccess 
		ifFalse: [ self error: 'Post failed: ', response contents printString ].

	json := NeoJSONReader fromString: response contents.
	
	^json
! !

!ZnOAuth1UserAccess methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/23/2012 15:31'!
accessToken
	^ accessToken! !

!ZnOAuth1UserAccess methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/23/2012 15:31'!
accessToken: anObject
	accessToken := anObject! !

!ZnOAuth1UserAccess methodsFor: 'accessing' stamp: 'jvds 1/3/2011 10:58'!
oauth1Service
	^ oauth1Service! !

!ZnOAuth1UserAccess methodsFor: 'accessing' stamp: 'jvds 1/3/2011 10:58'!
oauth1Service: anObject
	oauth1Service := anObject! !

Object subclass: #ZnSSOMessage
	instanceVariableNames: 'properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth1-Core'!

!ZnSSOMessage class methodsFor: 'constants' stamp: 'jvds 2/8/2011 10:58'!
assocTypeHmacSHA1
	"OpenID2 association type"
	
	^'HMAC-SHA1'! !

!ZnSSOMessage class methodsFor: 'instance creation' stamp: 'jvds 3/5/2010 11:01'!
newFromKeyValueForm: aString

	| eolChar msg stream |
	
	eolChar := (aString includes: Character lf)
		ifTrue: [ Character lf ]
		ifFalse: [ Character cr ].
	
	msg := self new.
	stream := ReadStream on: aString.
	[ stream atEnd ] whileFalse: [
		msg propertyAt: (stream upTo: $:) put: (stream upTo: eolChar)].
	^msg! !

!ZnSSOMessage class methodsFor: 'instance creation' stamp: 'jvds 11/6/2011 14:11'!
newFromRequest: aWARequest

	| msg stream |
	
	msg := self new.
	aWARequest fields keysAndValuesDo: [ :key :value | 
		(self propertyPrefix = '' or: [ key beginsWith: self propertyPrefix ])
			ifTrue: [ msg propertyAt: key put: value ] ].
	^msg! !

!ZnSSOMessage class methodsFor: 'constants' stamp: 'jvds 2/8/2011 10:57'!
nsAttributeExchangeExtension
	"Answer the namespace"

	^'http://openid.net/srv/ax/1.0'! !

!ZnSSOMessage class methodsFor: 'constants' stamp: 'jvds 2/8/2011 10:57'!
nsOAuth
	"Answer the namespace"

	^'http://specs.openid.net/extensions/oauth/1.0'! !

!ZnSSOMessage class methodsFor: 'constants' stamp: 'jvds 2/8/2011 10:57'!
nsOpenId2
	"Answer the namespace"

	^'http://specs.openid.net/auth/2.0'! !

!ZnSSOMessage class methodsFor: 'constants' stamp: 'jvds 2/8/2011 10:57'!
nsPape
	"Answer the namespace"

	^'http://specs.openid.net/extensions/pape/1.0'! !

!ZnSSOMessage class methodsFor: 'constants' stamp: 'jvds 11/6/2011 14:08'!
propertyPrefix

	^ ''! !

!ZnSSOMessage class methodsFor: 'utility' stamp: 'JanVanDeSandt 12/23/2012 15:55'!
signStringSHA1: aString usingSecret: aSecret 
	"Signs a string with <aSecret>, and encode the
	resulting byteArray into plain ASCII with base64."
	
	| hmac digest |

	hmac := SHA1 new hmac.
	hmac key: aSecret asByteArray.
	digest := hmac digestMessage: aString asByteArray.
	^ZnBase64Encoder new encode: digest! !

!ZnSSOMessage methodsFor: 'actions' stamp: 'emm 2/5/2012 18:45'!
addToUrl: anUrl
	"Add the receivers properties as parameters to ==anUrl==."
	
	self properties keysAndValuesDo: [ :key :value |
		anUrl queryAt: key put: value ]! !

!ZnSSOMessage methodsFor: 'private' stamp: 'jvds 3/2/2010 11:12'!
errorPropertyNotFound: aSelector 
	self error: 'Property ' , aSelector , ' not found.'! !

!ZnSSOMessage methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/24/2012 10:43'!
initialize

	super initialize.
	properties := Dictionary new! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 2/14/2011 12:54'!
prefixForNamespace: aNamespace ifAbsent: aBlock

	| begin |
	
	begin := 'openid.ns.'.
	self properties keysAndValuesDo: [ :key :value |
		(aNamespace = value and: [ (key copyFrom: 1 to: begin size) = begin ])
			ifTrue: [ ^key copyFrom: begin size + 1 to: key size ]].
	^aBlock value! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 3/2/2010 11:10'!
properties
	"Answer the property dictionary of the receiver."

	^properties! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 3/2/2010 11:11'!
propertyAt: aKey
	"Answer the value of the property ==aKey==, raises an error if the property doesn't exist."

	^ self propertyAt: aKey ifAbsent: [ self errorPropertyNotFound: aKey ]! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 3/2/2010 11:11'!
propertyAt: aKey ifAbsent: aBlock
	"Answer the value of the property ==aKey==, or the result of ==aBlock== if the property doesn't exist."

	^ self properties at: aKey ifAbsent: aBlock! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 3/2/2010 11:11'!
propertyAt: aKey ifAbsentPut: aBlock
	"Answer the value of the property ==aKey==, or if the property doesn't exist adds and answers the result of evaluating ==aBlock==."

	^ self properties at: aKey ifAbsentPut: aBlock! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 3/2/2010 11:11'!
propertyAt: aKey ifPresent: aBlock
	"Lookup the property ==aKey==, if it is present, answer the value of evaluating ==aBlock== block with the value. Otherwise, answer ==nil==."

	^ self properties at: aKey ifPresent: aBlock! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 3/2/2010 11:12'!
propertyAt: aKey put: aValue
	"Adds or replaces the property ==aKey== with ==aValue==."

	^ self properties at: aKey put: aValue! !

!ZnSSOMessage methodsFor: 'accessing' stamp: 'jvds 2/10/2011 21:15'!
removePropertyAt: aKey

	^ self properties removeKey: aKey ifAbsent: [ ]! !
